//=============================================================================
// Copyright 2006-2010 Daniel W. Dyer
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//=============================================================================
/**
 * This package provides a number of classes to simplify the development of
 * Swing GUIs for evolutionary programs.  As well as common controls for
 * modifying evolution parameters, it includes a Swing implementation of the
 * {@link org.uncommons.watchmaker.framework.interactive.Console} interface
 * for interactive evolutionary algorithms.
 */
package org.uncommons.watchmaker.swing;
